--[[
Author:  Javek
ReleaseDate: 3-24-2007
--]]


GnomeCruncher = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceDB-2.0","AceDebug-2.0")

function GnomeCruncher:OnInitialize()
    -- Called when the addon is enabled
    
	self:SetDebugging(false)
	self:SetDebugLevel(3)
   
	self:RegisterDB("GnomeCruncherDB")
	self:RegisterDefaults("profile", {
	   xpos = 400,
	   ypos = 400,
	   showing = true,
	   width = 150,
	   vertDir = true,
	   horizDir = true,
	   resR = 0.2,
	   resG = 0.6,
	   resB = 1,
      roundDigits = 3
	})
	--[[
		vertDir	true is up
		horizDir true is left
		resX are the colors for results
	--]]
	local options = { 
    type='group',
    args = {
	    show = {
	        	type = 'toggle',
	        	name = 'Show/Hide GnomeCruncher',
	        	desc = 'Shows or hides the calculator',
				get = 'isShown',
				set = 'show',
				map = {[false] = 'Hidden',[true] = 'Showing'}
	        	},
	    reset = {
	        	type = 'execute',
	        	name = "Resets GnomeCruncher's position",
	        	desc = 'Resets the calculator position',
	        	func = 'reset'
	        	},
	    width = {
	    		type = 'range',
	    		name = 'GnomeCruncher Width',
	    		desc = 'Sets the width of the calculator',
	    		get = 'getWidth',
	    		set = 'setWidth',
	    		min = 10,
	    		max = 1000,
	    		step = 1
	    		},
      roundDigits = {
         type='range',
         name='Round Digits',
         desc='The number of digits to round off to.',
         get = 'getRoundDigits',
         set = 'setRoundDigits',
         min = 0,
         max = 20,
         step = 1
      },
	    vertical = {
	    		type = 'toggle',
	    		name = 'Toggle Above/Bellow',
	    		desc = 'Toggles the vertical direction results grow in',
	    		get = 'getVertical',
	    		set = 'setVertical',
	    		map = {[false] = 'Down',[true]='Up'}
	    		},
	    side = {
	    		type = 'toggle',
	    		name = 'Toggle Left/Right',
	    		desc = 'Toggles whether bars line up on the left or right side',
	    		get = 'getHorizontal',
	    		set = 'setHorizontal',
	    		map = {[false] = 'Right',[true]='Left'}
	    		},
	    resultColor = {
	    		type = 'color',
	    		name = 'Results Text Color',
	    		desc = 'Sets the result text color',
	    		get = 'getResultColor',
	    		set = 'setResultColor',
	    		hasAlpha = false
	    		},
	    help = {
	    		type = 'execute',
	    		name = 'GnomeCruncher Help',
	    		desc = 'Displays GnomeCruncher Instructions.',
	    		func = 'help'
	    		}	    		
    }}    
        
   self:RegisterChatCommand({"/gnomecruncher"}, options)
	self.AceOO = AceLibrary("AceOO-2.0")   
end

function GnomeCruncher:OnEnable()
	self.newEntry = true
	self.result = nil
	self.resultColorText = nil
	
   --All letters will be prefixed with GnomeCruncher.objects.
   self.objects = {
      e = math.exp(1),
      pi = math.pi,
      ln = log,
      log = log10,
      cos = math.cos,
      sin = math.sin,
      tan = math.tan,
      acos = math.acos,
      asin = math.asin,
      atan = math.atan,      
      ans = 0
   }
	
	self:GuiInit()
	self:updateResultColorText()
	self:Debug("GnomeCruncher Enabled")
end

function GnomeCruncher:OnDisable()
    self.mainFrame:Hide()
end

function GnomeCruncher:newChar()
	--arg1 = char
   --Puts ans in front when starting typing with a math operation
	if self.newEntry then
		self:Debug("FIRST CHAR")	
		for i,j in pairs({"+","-","*","/","^"}) do
			if arg1 == j then
				self.entryFrame:SetText("ans"..arg1)
			end
		end
	end
	self.newEntry = false	
end

function GnomeCruncher:execute()
	--Execute Code and add result frame
	
	--Get Frame
	local f = nil
	local n = table.getn(self.frameStack)
	if n > 0 then
		f = self.frameStack[n]
		table.remove(self.frameStack)
	else
		f = CreateFrame("EditBox",nil,self.mainFrame,"GnomeCruncherResultBoxTemplate")	
	end
	
	-- Parse Text
	local origText = self.entryFrame:GetText()
	local text = origText:lower()
	self.entryFrame:SetText("")
	self.newEntry = true
   --Trim string
   text = string.gsub(text, "^%s*(.-)%s*$", "%1")
   text = string.gsub(text, "%a[%w_]*", "GnomeCruncher.objects.%0")
   
   local resultObject = nil
	-- Execute Calculation
   if string.find(text,"GnomeCruncher%.objects%.%a[%w_]*%s*=",1) == 1 then
      --don't do anything
      local s,e = string.find(text,"%a[%w_]*",23)
      resultObject = string.sub(text,s,e)
   else
      text = "GnomeCruncher.result="..text
   end
   
	self:Debug("Executing Text: "..text)
   
	local resultFunc = loadstring(text)
	local result
	if resultFunc == nil then
		result = self:errorText()
	else
		if pcall(resultFunc) then
			--No Errors
         result = resultObject and GnomeCruncher.objects[resultObject] or GnomeCruncher.result         
			if result == nil then
				result = self:errorText()
			else
			   self.objects.ans = result
            result=string.format("%."..self:getRoundDigits().."f",result)
            --Remove trailing zeros
            result = string.gsub(result,"%.?0*$","")            
				result = self.resultColorText..result
			end			
		else
			result = self:errorText()
		end
	end

	f:SetText(origText.." = "..result)
	
	-- Add this frame to the list and update position
	table.insert(self.resultFrames,f)
	local n = table.getn(self.resultFrames)
	self:updateFramePosition(n)
	self:updateFramePosition(n-1)
	f:Show()
end

function GnomeCruncher:errorText()
	return "|cFFFF0000Error!"
end

function GnomeCruncher:removeResult(removeFrame)
	self:Debug("Removing Result")
	for n,f in pairs(self.resultFrames) do
		if f == removeFrame then
			f:ClearAllPoints()
			f:SetText("")
			f:Hide()
			table.insert(self.frameStack,f)
			table.remove(self.resultFrames,n)
			self:updateFramePosition(n-1)
		end
	end
end


--Commands
function GnomeCruncher:isShown()
	return self.mainFrame:IsVisible()
end

function GnomeCruncher:show(state)
	if state then
		self.mainFrame:Show()
		self.db.profile.showing = true
	else
		self.mainFrame:Hide()
		self.db.profile.showing = false
	end
end

function GnomeCruncher:getRoundDigits()
   return self.db.profile.roundDigits
end
function GnomeCruncher:setRoundDigits(x)
   self.db.profile.roundDigits = x
end

function GnomeCruncher:getVertical()
	return self.db.profile.vertDir
end
function GnomeCruncher:setVertical(x)
	self.db.profile.vertDir = x
	for i = table.getn(self.resultFrames),1,-1 do
		self:updateFramePosition(i)
	end
end
function GnomeCruncher:getHorizontal()
	return self.db.profile.horizDir
end
function GnomeCruncher:setHorizontal(x)
	self.db.profile.horizDir = x
	self.entryFrame:ClearAllPoints()
	if x then
		--Left
		self.entryFrame:SetPoint("TOPRIGHT",self.mainFrame,"TOPLEFT",0,3)
	else
		self.entryFrame:SetPoint("TOPLEFT",self.mainFrame,"TOPRIGHT",0,3)
	end
	for i = table.getn(self.resultFrames),1,-1 do
		self:updateFramePosition(i)
	end	
end

function GnomeCruncher:getWidth()
	return self.db.profile.width
end
function GnomeCruncher:setWidth(x)
	self.db.profile.width = x
	self.entryFrame:SetWidth(x)
end

function GnomeCruncher:setResultColor(r,g,b)
	self:LevelDebug(3,"ResultColor("..r..","..g..","..b..")")
	self.db.profile.resR = r
	self.db.profile.resG = g
	self.db.profile.resB = b
	self:updateResultColorText()
end
function GnomeCruncher:getResultColor()
	return self.db.profile.resR,self.db.profile.resG,self.db.profile.resB
end
function GnomeCruncher:updateResultColorText()
	--3CA4DB
	self.resultColorText = ("|c%02x%02x%02x%02x"):format(255,self.db.profile.resR*255,self.db.profile.resG*255,self.db.profile.resB*255)
	self.resultColorText:upper()
	self:LevelDebug(3,"New Color Text: "..self.resultColorText:sub(3,-1).."TestText")
--	self.resultColorText = string.format("|%s%s%s",self.db.profile.resR, self.db.profile.resG, self.db.profile.resB)
end

function GnomeCruncher:help()
	self:Print([[Instructions:
Pressing Enter performs a calculation.
The following mathematical operations are available:
Addition: +  Subtraction: -  Multiplication: *  Division: /  Exponents: ^ Modulous: %
ln(), log(), cos(), sin(), tan(), acos(), asin(), atan(), e, pi
Operations can be grouped with parenthesis.
The word ans is set to the value of the last result.
You can save your own variables and use them later, for example: a1 = 12
Note: Custom variables must start with a letter
Various configuration options can be set with /gnomecruncher]])
end
